package tech.krave.twilio_chat_plugin

import com.twilio.chat.*
import com.twilio.chat.ChannelListener as ChListener

class ChannelListener : BaseListener(), ChListener {
    override fun onMemberDeleted(member: Member?) {
        TwilioChatPlugin.debug("""ChannelListener.onMemberDeleted => memberSID : ${member?.sid}""")
        sendEvent("memberDeleted", mapOf("member" to SerializationHelper.memberToMap(member)))
    }

    override fun onTypingEnded(channel: Channel?, member: Member?) {
        TwilioChatPlugin.debug("""
            |ChannelListener.onTypingEnded => 
            |memberSID : ${member?.sid}
            |channelSID: ${channel?.sid}
            |""".trimMargin())
        sendEvent("typingEnded", mapOf(
                "member" to SerializationHelper.memberToMap(member),
                "channel" to SerializationHelper.channelToMap(channel)
        ))
    }

    override fun onMessageAdded(message: Message?) {
        TwilioChatPlugin.debug("""ChannelListener.onMessageAdded => messageSID : ${message?.sid}""")
        sendEvent("messageAdded", mapOf("message" to SerializationHelper.messageToMap(message)))
    }

    override fun onMessageDeleted(message: Message?) {
        TwilioChatPlugin.debug("""ChannelListener.onMessageDeleted => messageSID : ${message?.sid}""")
        sendEvent("messageDeleted", mapOf("message" to SerializationHelper.messageToMap(message)))
    }

    override fun onMemberAdded(member: Member?) {
        TwilioChatPlugin.debug("""ChannelListener.onMemberAdded => memberSID : ${member?.sid}""")
        sendEvent("memberAdded", mapOf("member" to SerializationHelper.memberToMap(member)))
    }

    override fun onTypingStarted(channel: Channel?, member: Member?) {
        TwilioChatPlugin.debug("""
            |ChannelListener.onTypingStarted => 
            |memberSID : ${member?.sid}
            |channelSID: ${channel?.sid}
            |""".trimMargin())
        sendEvent("typingStarted", mapOf(
                "member" to SerializationHelper.memberToMap(member),
                "channel" to SerializationHelper.channelToMap(channel)
        ))
    }

    override fun onSynchronizationChanged(channel: Channel?) {
        TwilioChatPlugin.debug("""
            |ChannelListener.onSynchronizationChanged => 
            |channelSID: ${channel?.sid}
            |""".trimMargin())
        sendEvent("channelSynchronizationChanged",
                mapOf("channel" to SerializationHelper.channelToMap(channel))
        )
    }

    override fun onMessageUpdated(message: Message?, messageUpdateReason: Message.UpdateReason?) {
        TwilioChatPlugin.debug("""ChannelListener.onMessageUpdated => messageSID : ${message?.sid}""")
        sendEvent("messageUpdated", mapOf("message" to SerializationHelper.messageToMap(message),
                "updateReason" to SerializationHelper.messageUpdatedReasonToMap(messageUpdateReason)
        ))
    }

    override fun onMemberUpdated(member: Member?, memberUpdateReason: Member.UpdateReason?) {
        TwilioChatPlugin.debug("""ChannelListener.onMemberUpdated => memberSID : ${member?.sid}""")
        sendEvent("memberUpdated", mapOf("member" to SerializationHelper.memberToMap(member),
                "updateReason" to SerializationHelper.memberUpdatedReasonToMap(memberUpdateReason)
        ))
    }

}