package tech.krave.twilio_chat_plugin

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.twilio.chat.Channel
import com.twilio.chat.ChannelDescriptor
import com.twilio.chat.ChatClient
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.flutter.plugin.platform.PlatformViewRegistry

/** TwilioChatPlugin */
public class TwilioChatPlugin : FlutterPlugin {

    private lateinit var methodChannel: MethodChannel

    private lateinit var chatClientChannel: EventChannel

    private lateinit var loggingChannel: EventChannel

    private lateinit var channelChannel: EventChannel


    companion object {
        @Suppress("unused")
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val instance = TwilioChatPlugin()
            instance.onAttachedToEngine(registrar.context(), registrar.messenger(), registrar.platformViewRegistry())
        }

        @JvmStatic
        val LOG_TAG = "twilio_chat_plugin"

        var loggingSink: EventChannel.EventSink? = null

        var nativeDebug: Boolean = true

        var chatClientListener: ChatClientListener? = ChatClientListener()

        var channelListener: ChannelListener? = ChannelListener()

        var chatClient: ChatClient? = null

        var channel: Channel? = null

        var userChannelDescriptors: ArrayList<ChannelDescriptor>? = arrayListOf()

        var publicChannelDescriptors: ArrayList<ChannelDescriptor>? = arrayListOf()

        @JvmStatic
        fun debug(msg: String) {
            if (nativeDebug) {
                Log.d(LOG_TAG, msg)
                loggingSink?.success(msg)
            }
        }

    }

    override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        onAttachedToEngine(binding.applicationContext, binding.binaryMessenger, binding.platformViewRegistry)
    }

    private fun onAttachedToEngine(applicationContext: Context, messenger: BinaryMessenger, platformViewRegistry: PlatformViewRegistry) {
        val pluginHandler = PluginHandler(applicationContext)
        methodChannel = MethodChannel(messenger, "krave.tech/twilio_chat_plugin")
        methodChannel.setMethodCallHandler(pluginHandler)

        chatClientChannel = EventChannel(messenger, "krave.tech/twilio_chat_plugin/chat_client")
        chatClientChannel.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink) {
                debug("TwilioChatPlugin.onAttachedToEngine => ChatClient eventChannel attached")
                chatClientListener?.events = events
            }

            override fun onCancel(arguments: Any?) {
                debug("TwilioChatPlugin.onAttachedToEngine => ChatClient eventChannel detached")
                chatClientListener?.events = null
            }
        })

        channelChannel = EventChannel(messenger, "krave.tech/twilio_chat_plugin/channel")
        channelChannel.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink) {
                debug("TwilioChatPlugin.onAttachedToEngine => Channel eventChannel attached")
                channelListener?.events = events
            }

            override fun onCancel(arguments: Any?) {
                debug("TwilioChatPlugin.onAttachedToEngine => Channel eventChannel detached")
                channelListener?.events = null
            }
        })

        loggingChannel = EventChannel(messenger, "krave.tech/twilio_chat_plugin/logging")
        loggingChannel.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink) {
                debug("TwilioChatPlugin.onAttachedToEngine => Logging eventChannel attached")
                loggingSink = events
            }

            override fun onCancel(arguments: Any?) {
                debug("TwilioChatPlugin.onAttachedToEngine => Logging eventChannel detached")
                loggingSink = null
            }
        })

    }


    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel.setMethodCallHandler(null)
        chatClientChannel.setStreamHandler(null)
        loggingChannel.setStreamHandler(null)
        channelChannel.setStreamHandler(null)
    }


}
