//
//  ChannelListener.swift
//  twilio_chat_plugin
//
//  Created by Austin Goh on 21/04/2020.
//

import Foundation
import TwilioChatClient

class ChannelListener: BaseListener, TCHChannelDelegate{

    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, updated: TCHChannelUpdate) {
        SwiftTwilioChatPlugin.debug("ChannelListener.channelUpdated => channel sid is '\(String(describing: channel.sid))'")
        channel.members?.members(){
            res, members in
            var membersListResult = [[String:Any]]()
            for member in (members?.items())! {
                membersListResult.append(SerializationHelper.memberToDict(member))
            }
            self.sendEvent("channelUpdated", data: ["channel": SerializationHelper.channelToDict(channel, membersListResult), "updateReason": SerializationHelper.channelUpdateToDict(updated)])
        }
        
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, member: TCHMember, userSubscribed user: TCHUser) {
        SwiftTwilioChatPlugin.debug("ChannelListener.userSubscribed => user is '\(String(describing: user.identity))'")
        sendEvent("channelUserSubscribed", data: ["user": SerializationHelper.userToDict(user)])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, member: TCHMember, userUnsubscribed user: TCHUser) {
        SwiftTwilioChatPlugin.debug("ChannelListener.userUnsubscribed => user is '\(String(describing: user.identity))'")
        sendEvent("channelUserUnsubscribed", data: ["user": SerializationHelper.userToDict(user)])
    }

    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, member: TCHMember, user: TCHUser, updated: TCHUserUpdate) {
        SwiftTwilioChatPlugin.debug("ChannelListener.userUpdated => user is '\(user.identity ?? "")'")
        sendEvent("channelUserUpdated", data: [ "user": SerializationHelper.userToDict(user),
                                         "updateReason": SerializationHelper.userUpdateToDict(updated)])
    }
    
    func chatClient(_ client: TwilioChatClient, channelDeleted channel: TCHChannel) {
        SwiftTwilioChatPlugin.debug("ChannelListener.channelDeleted => channel sid is '\(String(describing: channel.sid))'")
        sendEvent("channelDeleted", data: [ "channel": SerializationHelper.channelToDict(channel, [[String:Any]]()) ])
    }

    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, synchronizationStatusUpdated status: TCHChannelSynchronizationStatus) {
        SwiftTwilioChatPlugin.debug("""
            ChannelListener.channelAdded => channelSID : '\(String(describing: channel.sid))'${channel?.sid}
        """
        )
        sendEvent("channelAdded",data: [
            "channel": SerializationHelper.channelToDict(channel, [[String:Any]]()),
            "status": SerializationHelper.channelSyncStatusToDict(status)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, memberLeft member: TCHMember) {
        SwiftTwilioChatPlugin.debug("ChannelListener.memberLeft => member is '\(member.identity ?? "")'")
        sendEvent("memberDeleted", data: [
            "member": SerializationHelper.memberToDict(member)
        ])
    }

    func chatClient(_ client: TwilioChatClient, typingEndedOn channel: TCHChannel, member: TCHMember) {
        SwiftTwilioChatPlugin.debug("""
            ChannelListener.typingEndedOn =>
            memberSID : '\(member.sid ?? "")'
            channelSID : '\(channel.sid ?? "")'
        """)
        sendEvent("typingEnded", data: [
            "channel": SerializationHelper.channelToDict(channel, [[String: Any]]()),
            "member": SerializationHelper.memberToDict(member)
        ])

    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, messageAdded message: TCHMessage) {
        SwiftTwilioChatPlugin.debug("ChannelListener.messageAdded => messageSID : '\(message.sid ?? "")'")
        sendEvent("messageAdded", data: [
            "message": SerializationHelper.messageToDict(message)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, messageDeleted message: TCHMessage) {
        SwiftTwilioChatPlugin.debug("ChannelListener.messageDeleted => messageSID : '\(message.sid ?? "")'")
        sendEvent("messageDeleted", data: [
            "message": SerializationHelper.messageToDict(message)
        ])
        
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, memberJoined member: TCHMember) {
        SwiftTwilioChatPlugin.debug("ChannelListener.memberJoined => memberSID : '\(member.sid ?? "")'")
        sendEvent("memberAdded", data: [
            "member": SerializationHelper.memberToDict(member)
        ])
    }

    func chatClient(_ client: TwilioChatClient, typingStartedOn channel: TCHChannel, member: TCHMember) {
        SwiftTwilioChatPlugin.debug("""
            ChannelListener.typingStartedOn =>
            memberSID : '\(member.sid ?? "")'
            channelSID : '\(channel.sid ?? "")'
        """)
        sendEvent("typingStarted", data: [
            "channel": SerializationHelper.channelToDict(channel, [[String:Any]]()),
            "member": SerializationHelper.memberToDict(member)
        ])
    }

    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, message: TCHMessage, updated: TCHMessageUpdate) {
        SwiftTwilioChatPlugin.debug("ChannelListener.messageUpdated => messageSID : '\(message.sid ?? "")'")
        sendEvent("messageUpdated", data: [
            "message": SerializationHelper.messageToDict(message),
            "updateReason": SerializationHelper.messageUpdateToDict(updated)
        ])
    }

    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, member: TCHMember, updated: TCHMemberUpdate) {
        SwiftTwilioChatPlugin.debug("ChannelListener.memberUpdated => memberSID : '\(member.sid ?? "")'")
        sendEvent("memberUpdated", data: [
            "member": SerializationHelper.memberToDict(member),
            "updateReason": SerializationHelper.memberUpdateToDict(updated)
        ])
    }

}
