//
//  Models.swift
//  twilio_chat_plugin
//
//  Created by Austin Goh on 27/04/2020.
//

import Foundation
import TwilioChatClient


class SerializationHelper {
    static func connectionStateToDict(_ connectionState: TCHClientConnectionState)-> [String: Any] {
            var connectionStateMsg: String
            switch connectionState {
                case TCHClientConnectionState.connecting:
                    connectionStateMsg = "CONNECTING"
                case TCHClientConnectionState.connected:
                    connectionStateMsg = "CONNECTED"
                case TCHClientConnectionState.denied:
                    connectionStateMsg = "DENIED"
                case TCHClientConnectionState.disconnected:
                    connectionStateMsg = "DISCONNECTED"
                default:
                    connectionStateMsg = "UNKNOWN"
            }
            return [
                "value": connectionState.rawValue,
                "name": connectionStateMsg
            ]
        }
        
    static func chatClientSyncStatusToDict(_ chatClientSyncStatus: TCHClientSynchronizationStatus)-> [String: Any] {
            var chatClientSyncStatusMsg: String
            switch chatClientSyncStatus {
            case TCHClientSynchronizationStatus.started:
                chatClientSyncStatusMsg = "STARTED"
            case TCHClientSynchronizationStatus.completed:
                chatClientSyncStatusMsg = "COMPLETED"
            case TCHClientSynchronizationStatus.channelsListCompleted:
                chatClientSyncStatusMsg = "CHANNELS_LIST_COMPLETED"
            case TCHClientSynchronizationStatus.failed:
                chatClientSyncStatusMsg = "FAILED"
            default:
                chatClientSyncStatusMsg = "UNKNOWN"
            }
            return [
                "value": chatClientSyncStatus.rawValue,
                "name": chatClientSyncStatusMsg
            ]
        }
        
     static func channelSyncStatusToDict(_ channelSyncStatus: TCHChannelSynchronizationStatus)-> [String: Any] {
            var channelSyncStatusMsg: String
            switch channelSyncStatus {
            case TCHChannelSynchronizationStatus.all:
                channelSyncStatusMsg = "ALL"
            case TCHChannelSynchronizationStatus.identifier:
                channelSyncStatusMsg = "IDENTIFIER"
            case TCHChannelSynchronizationStatus.metadata:
                channelSyncStatusMsg = "METADATA"
            case TCHChannelSynchronizationStatus.failed:
                channelSyncStatusMsg = "FAILED"
            case TCHChannelSynchronizationStatus.none:
                       channelSyncStatusMsg = "NONE"
            default:
                channelSyncStatusMsg = "UNKNOWN"
            }
            return [
                "value": channelSyncStatus.rawValue,
                "name": channelSyncStatusMsg
            ]
        }
        
        static func membersToDict(_ members: TCHMembers) -> [String: Any] {
            return [
                "membersList": members
            ]
        }
        
        static func memberToDict(_ member: TCHMember) -> [String: Any] {
            return [
                "identity": member.identity ?? "",
                "sid": member.sid ?? "",
                "lastConsumptionTimestamp": member.lastConsumptionTimestamp,
                "lastConsumptionIndex": member.lastConsumedMessageIndex,
            ]
        }
        
        static func errorToDict(_ error: TCHError) -> [String: Any] {
            return [
                "code": error.code,
                "message": error.description,
                "status": "",
            ]
        }
        
        static func userUpdateToDict(_ userUpdate: TCHUserUpdate) -> [String: Any] {
            var userUpdateMessage = ""
            switch userUpdate {
            case TCHUserUpdate.attributes:
                userUpdateMessage = "ATTRIBUTES"
            case TCHUserUpdate.friendlyName:
                userUpdateMessage = "FRIENDLY_NAME"
            case TCHUserUpdate.reachabilityNotifiable:
                userUpdateMessage = "REACHABILITY_NOTIFIABLE"
            case TCHUserUpdate.reachabilityOnline:
                userUpdateMessage =  "REACHABILITY_ONLINE"
            default:
                userUpdateMessage = "UNKNOWN"
            }
            return [
                "value": userUpdate.rawValue,
                "name": userUpdateMessage,
           ]
       }
        
        static func messageUpdateToDict(_ messageUpdate: TCHMessageUpdate) -> [String: Any] {
            var messageUpdateMessage = ""
            switch messageUpdate {
                case TCHMessageUpdate.attributes:
                    messageUpdateMessage = "ATTRIBUTES"
                case TCHMessageUpdate.body:
                    messageUpdateMessage = "BODY"
            default:
                messageUpdateMessage = "UNKNOWN"
            }
            return [
                "value":    messageUpdate.rawValue,
                "name": messageUpdateMessage
            ]
        }
        
        static func memberUpdateToDict(_ memberUpdate: TCHMemberUpdate) -> [String: Any]{
            var memberUpdateMessage = ""
            switch memberUpdate {
            case TCHMemberUpdate.attributes:
                memberUpdateMessage = "ATTRIBUTES"
            case TCHMemberUpdate.lastConsumedMessageIndex:
                memberUpdateMessage = "LAST_CONSUME_MESSAGE_INDEX"
            default:
                memberUpdateMessage = "UNKNOWN"
            }
            return [
                "value": memberUpdate.rawValue,
                "name": memberUpdateMessage
            ]
        }
        
         static func channelUpdateToDict(_ channelUpdate: TCHChannelUpdate) -> [String: Any] {
             var channelUpdateMessage = ""
             switch channelUpdate {
             case TCHChannelUpdate.attributes:
                 channelUpdateMessage = "ATTRIBUTES"
             case TCHChannelUpdate.friendlyName:
                 channelUpdateMessage = "FRIENDLY_NAME"
             case TCHChannelUpdate.lastConsumedMessageIndex:
                 channelUpdateMessage = "LAST_CONSUMED_MESSAGE_INDEX"
             case TCHChannelUpdate.lastMessage:
                 channelUpdateMessage =  "LAST_MESSAGE"
             case TCHChannelUpdate.status:
                channelUpdateMessage =  "STATUS"
             default:
                 channelUpdateMessage = "UNKNOWN"
             }
             return [
                 "value": channelUpdate.rawValue,
                 "name": channelUpdateMessage,
            ]
        }
    
    static func channelToDict(_ channel: TCHChannel?,_ membersList: [[String: Any]] )-> [String: Any] {
            var channelType: Dictionary = [String: Any]()
            if(channel?.type != nil ){
                channelType = channelTypeToDict(channelType: channel!.type)
            }
            return [
                "sid": channel?.sid as Any,
                "createdBy": channel?.createdBy as Any,
                "dateCreated": channel?.dateCreated as Any,
                "dateUpdated": channel?.dateUpdated as Any,
                "friendlyName": channel?.friendlyName as Any,
                "uniqueName": channel?.uniqueName as Any,
                "type": channelType,
                "members": [
                    "membersList": membersList
                ],
            ]
        }
           
        static func messageToDict(_ message: TCHMessage) -> [String: Any] {
            return [
                "author" : message.author ?? "",
    //            "channelSID" : "",
                "dateCreated" : message.dateUpdated ?? "",
                "memberSID" : message.memberSid ?? "",
                "messageBody" : message.body ?? "",
                "sid" : message.sid ?? ""
            ]
        }
        
        static func channelDescriptorToDict(_ channel: TCHChannelDescriptor?) -> [String: Any] {
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = "yyyy-MM-dd'T'HH:mm:ss.SSSXXX"
            return [
                "sid" : channel?.sid ?? "",
                "createdBy" : channel?.createdBy ?? "" ,
                "dateCreated" : dateFormatter.string(from: channel?.dateCreated ?? Date())  as Any,
                "dateUpdated" : dateFormatter.string(from: channel?.dateUpdated ?? Date())  as Any,
                "friendlyName" : channel?.friendlyName ?? "" ,
                "uniqueName" : channel?.uniqueName ?? "" ,
                "membersCount" : channel?.membersCount() as Any ,
                "messagesCount" : channel?.messagesCount() as Any ,
                "unconsumedMessagesCount" : 0
            ]
        }
        
    static func chatClientToDict(_ chatClient: TwilioChatClient)-> [String: Any] {
            var users: Dictionary = [String: Any]()
            var channels: Dictionary = [String: Any]()
            if chatClient.users() != nil {
                users = usersToDict(users: chatClient.users(), user: chatClient.user)
            }
            if chatClient.channelsList() != nil {
                channels = channelsToDict(channels: chatClient.channelsList())
            }
            return [
                "connectState" : chatClientConnectionStatusToDict(connectionState: chatClient.connectionState) as Any,
                "isReachabilityEnabled" : chatClient.isReachabilityEnabled() as Any,
                "myIdentity" : chatClient.user?.identity ?? "" as Any,
                "user": userToDict((chatClient.user!)) as Any,
                "users" :  users as Any,
                "channels" :  channels as Any,
                "properties" : [:] as Any
            ]
        }
        
        static func chatClientConnectionStatusToDict(connectionState: TCHClientConnectionState) ->  [String: Any] {
            var chatClientConnectionStateMessage = ""
            switch connectionState {
                case TCHClientConnectionState.connected:
                    chatClientConnectionStateMessage = "CONNECTED"
                case TCHClientConnectionState.connecting:
                    chatClientConnectionStateMessage = "CONNECTING"
                case TCHClientConnectionState.denied:
                    chatClientConnectionStateMessage = "DENIED"
                case TCHClientConnectionState.disconnected:
                    chatClientConnectionStateMessage = "DISCONNECTED"
                case TCHClientConnectionState.error:
                    chatClientConnectionStateMessage = "ERROR"
                case TCHClientConnectionState.fatalError:
                    chatClientConnectionStateMessage = "FATAL_ERROR"
                default:
                    chatClientConnectionStateMessage = "UNKNOWN"
            }
            return [
                "name" : chatClientConnectionStateMessage,
                "value" : connectionState.rawValue
            ]
        }
        
        static func usersToDict(users: TCHUsers?, user: TCHUser?) -> [String: Any] {
            var subscribedUsersResult = [[String: Any]]()
            if(users != nil){
                for usr in users!.subscribedUsers() {
                    subscribedUsersResult.append(userToDict(usr))
                }
            }
            
            return [
                "myUser" : userToDict(user!),
                "subscribedUsers" : subscribedUsersResult
            ]
        }
        
        static func userToDict(_ user: TCHUser)-> [String: Any] {
            
            return [
                "identity": user.identity ?? "",
                "friendlyName": user.friendlyName ?? "",
                "isOnline": user.isOnline(),
                "isNotifiable": user.isNotifiable(),
                "isSubscribed": user.isSubscribed()
            ]
        }
        
        static func channelsToDict(channels: TCHChannels?)-> [String:Any] {
            var subscribedChannelsResult = [[String: Any]]()
            if(channels != nil){
                
                for ch in channels!.subscribedChannels() {
                   subscribedChannelsResult.append(channelToDict(ch, [[String: Any]]()))
                }
            }
            
            return [
                "subscribedChannels" : subscribedChannelsResult as Any
            ]
        }
        
        static func chatClientPropertiesToDict(properties: TwilioChatClientProperties) ->[String: Any]{
            return [
                "region" : properties.region,
                "deferCA" : ""
            ]
        }
        
        static func channelTypeToDict(channelType: TCHChannelType) ->  [String: Any] {
            var channelTypeMessage = ""
            switch channelType {
                case TCHChannelType.public:
                    channelTypeMessage = "PUBLIC"
                case TCHChannelType.private:
                     channelTypeMessage = "PRIVATE"
                default:
                     channelTypeMessage = "UNKNOWN"
            }
            return [
                "name" :  channelTypeMessage,
                "value" : channelType.rawValue
            ]
        }
}
