import Flutter
import UIKit
import TwilioChatClient

public class SwiftTwilioChatPlugin: NSObject, FlutterPlugin {

    internal static var chatClientListener: ChatClientListener?
    
    internal static var channelListener: ChannelListener?
    
    internal static var chatClient: TwilioChatClient?
    
    internal static var channel: TCHChannel?
    
    internal static var deviceToken: Data?
    
    internal static var connected: Bool?
    
    internal static var channelsList:TCHChannels?
    
    internal static var userChannelDescriptors:NSMutableOrderedSet?
    
    internal static var publicChannelDescriptors:NSMutableOrderedSet?
    
    public static var loggingSink: FlutterEventSink?
    
    public static var nativeDebug = true
    
    public static func debug(_ msg: String) {
        if nativeDebug {
            NSLog(msg)
            guard let loggingSink = loggingSink else {
                return
            }
            loggingSink(msg)
        }
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = SwiftTwilioChatPlugin()
        instance.onRegister(registrar)
    }

    
    private var methodChannel: FlutterMethodChannel?

    private var chatClientChannel: FlutterEventChannel?
    
    private var channelChannel: FlutterEventChannel?

    private var loggingChannel: FlutterEventChannel?
    
    public func onRegister(_ registrar: FlutterPluginRegistrar) {
        let pluginHandler = PluginHandler()
        methodChannel = FlutterMethodChannel(name: "krave.tech/twilio_chat_plugin", binaryMessenger: registrar.messenger())
        methodChannel?.setMethodCallHandler(pluginHandler.handle)
        
        loggingChannel = FlutterEventChannel(name: "krave.tech/twilio_chat_plugin/logging", binaryMessenger: registrar.messenger())
        loggingChannel?.setStreamHandler(LoggingStreamHandler())
        
        chatClientChannel = FlutterEventChannel(name: "krave.tech/twilio_chat_plugin/chat_client", binaryMessenger: registrar.messenger())
        chatClientChannel?.setStreamHandler(ChatClientStreamHandler())
        
        channelChannel = FlutterEventChannel(name: "krave.tech/twilio_chat_plugin/channel", binaryMessenger: registrar.messenger())
        channelChannel?.setStreamHandler(ChannelStreamHandler())

    }
    
    class ChatClientStreamHandler: NSObject, FlutterStreamHandler {
        func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            guard let chatClientListener = SwiftTwilioChatPlugin.chatClientListener else { return nil }
            SwiftTwilioChatPlugin.debug("ChatClientStreamHandler.onListen => ChatClient eventChannel attached")
            chatClientListener.events = events
            return nil
        }

        func onCancel(withArguments arguments: Any?) -> FlutterError? {
            guard let chatClientListener = SwiftTwilioChatPlugin.chatClientListener else { return nil }
            SwiftTwilioChatPlugin.debug("ChatClientStreamHandler.onCancel => ChatClient eventChannel detached")
            
            chatClientListener.events = nil
            return nil
        }
    }
    
    class ChannelStreamHandler: NSObject, FlutterStreamHandler {
           func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            guard let channelListener = SwiftTwilioChatPlugin.channelListener else { return nil }
               SwiftTwilioChatPlugin.debug("ChannelStreamHandler.onListen => Channel eventChannel attached")
                
               channelListener.events = events
               return nil
           }

           func onCancel(withArguments arguments: Any?) -> FlutterError? {
            guard let channelListener = SwiftTwilioChatPlugin.channelListener else { return nil }
               SwiftTwilioChatPlugin.debug("ChannelStreamHandler.onCancel => Channel eventChannel detached")
               
               channelListener.events = nil
               return nil
           }
       }
       
    
    class LoggingStreamHandler: NSObject, FlutterStreamHandler {
        func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            SwiftTwilioChatPlugin.debug("LoggingStreamHandler.onListen => Logging eventChannel attached")
            SwiftTwilioChatPlugin.loggingSink = events
            return nil
        }

        func onCancel(withArguments arguments: Any?) -> FlutterError? {
            SwiftTwilioChatPlugin.debug("LoggingStreamHandler.onCancel => Logging eventChannel detached")
            SwiftTwilioChatPlugin.loggingSink = nil
            return nil
        }
    }

}
