part of twilio_chat_plugin;

class MemberDeletedEvent {
  final Member member;

  MemberDeletedEvent(
    this.member,
  ) : assert(member != null);
}

class TypingEndedEvent {
  final Channel channel;
  final Member member;

  TypingEndedEvent(
    this.channel,
    this.member,
  )   : assert(member != null),
        assert(channel != null);
}

class MessageAddedEvent {
  final Message message;

  MessageAddedEvent(this.message) : assert(message != null);
}

class MessageDeletedEvent {
  final Message message;

  MessageDeletedEvent(this.message) : assert(message != null);
}

class MemberAddedEvent {
  final Member member;

  MemberAddedEvent(
    this.member,
  ) : assert(member != null);
}

class TypingStartedEvent {
  final Channel channel;
  final Member member;

  TypingStartedEvent(
    this.channel,
    this.member,
  )   : assert(member != null),
        assert(channel != null);
}

class SynchronizationChangedEvent {
  final Channel channel;

  SynchronizationChangedEvent(this.channel) : assert(channel != null);
}

class MessageUpdatedEvent {
  final Message message;
  final MessageUpdateReason updateReason;

  MessageUpdatedEvent(this.message, this.updateReason)
      : assert(message != null),
        assert(updateReason != null);
}

class MemberUpdatedEvent {
  final Member member;
  final MemberUpdateReason updateReason;

  MemberUpdatedEvent(this.member, this.updateReason)
      : assert(member != null),
        assert(updateReason != null);
}

class ChannelDeletedEvent {
  final Channel channel;

  ChannelDeletedEvent(this.channel) : assert(channel != null);
}

class InvitedToChannelNotificationEvent {
  final String channelSID;

  InvitedToChannelNotificationEvent(this.channelSID)
      : assert(channelSID != null);
}

class ClientSynchronizationEvent {
  final ChatClientSynchronizationStatus syncStatus;

  ClientSynchronizationEvent(this.syncStatus) : assert(syncStatus != null);
}

class NotificationSubscribedEvent {
  NotificationSubscribedEvent();
}

class UserSubscribedEvent {
  final User user;

  UserSubscribedEvent(this.user) : assert(user != null);
}

class ChannelUpdatedEvent {
  final Channel channel;
  final ChannelUpdateReason updateReason;

  ChannelUpdatedEvent(
    this.channel,
    this.updateReason,
  )   : assert(channel != null),
        assert(updateReason != null);
}

class RemovedFromChannelNotificationEvent {
  final String channelSID;

  RemovedFromChannelNotificationEvent(this.channelSID)
      : assert(channelSID != null);
}

class NotificationFailedEvent {
  final ErrorInfo errorInfo;

  NotificationFailedEvent(this.errorInfo) : assert(errorInfo != null);
}

class TokenExpiredEvent {
  TokenExpiredEvent();
}

class ChannelJoinedEvent {
  final Channel channel;

  ChannelJoinedEvent(this.channel) : assert(channel != null);
}

class ChannelAddedEvent {
  final Channel channel;

  ChannelAddedEvent(this.channel) : assert(channel != null);
}

class ChannelSynchronizationChangedEvent {
  final Channel channel;

  ChannelSynchronizationChangedEvent(this.channel) : assert(channel != null);
}

class UserUnsubscribedEvent {
  final User user;

  UserUnsubscribedEvent(this.user) : assert(user != null);
}

class AddedToChannelNotificationEvent {
  final String channelSID;

  AddedToChannelNotificationEvent(this.channelSID) : assert(channelSID != null);
}

class ChannelInvitedEvent {
  final Channel channel;

  ChannelInvitedEvent(this.channel) : assert(channel != null);
}

class NewMessageNotificationEvent {
  final String channelSID;
  final String messageSID;
  final int messageIndex;

  NewMessageNotificationEvent(
      this.channelSID, this.messageSID, this.messageIndex)
      : assert(channelSID != null),
        assert(messageSID != null),
        assert(messageIndex != null);
}

class ConnectionStateChangedEvent {
  final ChatClientConnectionState connectionState;

  ConnectionStateChangedEvent(this.connectionState)
      : assert(connectionState != null);
}

class ErrorEvent {
  final ErrorInfo errorInfo;

  ErrorEvent(this.errorInfo) : assert(errorInfo != null);
}

class UserUpdatedEvent {
  final User user;
  final UserUpdateReason updateReason;

  UserUpdatedEvent(this.user, this.updateReason)
      : assert(user != null),
        assert(updateReason != null);
}

class TokenAboutToExpireEvent {
  TokenAboutToExpireEvent();
}
